package com.loafle.overflow;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.google.protobuf.ByteString;
import com.loafle.overflow.crawler.Crawler;
import com.loafle.overflow.crawler.config.Config;
import com.loafle.overflow.rpc.*;
import io.grpc.ManagedChannel;
import io.grpc.ManagedChannelBuilder;
import junit.framework.Assert;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;

import java.io.*;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.logging.Logger;

import static org.junit.Assert.assertEquals;

/**
 * Created by root on 17. 4. 13.
 */
public class ServerTest {
    private static final Logger logger = Logger.getLogger(ServerTest.class.getName());
    public class TestCrawlerImpl extends Crawler {
        @Override
        public Object getInternal(Config config) throws Exception {
            return config.getId();
        }
    }

    Server server;

    @Before
    public void setUp() throws Exception {
        server = new Server();
        server.addDelegate(Crawlers.REDIS.name(),new TestCrawlerImpl());
        server.start();
    }

    @After
    public void tearDown() throws Exception {
        server.stop();
    }


    public DataGrpc.DataBlockingStub newDataClient(String ip, int port) {
        return DataGrpc.newBlockingStub(newChannel(ip,port));
    }

    public ConfigGrpc.ConfigBlockingStub newConfigClient(String ip, int port) {
        return ConfigGrpc.newBlockingStub(newChannel(ip,port));
    }

    public ManagedChannel newChannel(String ip, int port) {
        return ManagedChannelBuilder.forAddress(ip,port).usePlaintext(true).build();
    }

    public Output init() throws IOException {
        ConfigGrpc.ConfigBlockingStub client = newConfigClient("localhost",50052);

        ClassLoader classLoader = getClass().getClassLoader();
        Path path = Paths.get(classLoader.getResource("config/example.json").getFile());
        byte[] data = Files.readAllBytes(path);
        ByteString sss = ByteString.copyFrom(data);

        Init input = Init.newBuilder().
                setData(sss).
                setName(Crawlers.REDIS).build();
        InputArray arr = InputArray.newBuilder().addIn(input).build();

        return client.init(arr);
    }


    @Test
    public void TestInit() throws IOException, ClassNotFoundException {

        Output out = init();

        ByteArrayInputStream bis = new ByteArrayInputStream(out.getData().toByteArray());
        ObjectInput in = new ObjectInputStream(bis);
        Object o = in.readObject();

        logger.info("!!!!!!!!!!!!1 " + o);
        assertEquals(true,o);

    }

    @Test
    public void TestAdd() throws Exception {
        init();

        ConfigGrpc.ConfigBlockingStub client = newConfigClient("localhost",50052);

        // path is test resources/config/example.json
        ClassLoader classLoader = getClass().getClassLoader();
        Path path = Paths.get(classLoader.getResource("config/example.json").getFile());
        byte[] data = Files.readAllBytes(path);
        ByteString sss = ByteString.copyFrom(data);

        InputAdd input = InputAdd.newBuilder().
                setData(sss).
                setName(Crawlers.REDIS).build();

        Output out = client.add(input);

        ByteArrayInputStream bis = new ByteArrayInputStream(out.getData().toByteArray());
        ObjectInput in = new ObjectInputStream(bis);
        Object o = in.readObject();


        logger.info("!!!!!!!!!!!!1 " + o);
        assertEquals(true,o);
    }


    @Test
    public void TestRemove() throws  Exception {
        init();

        ConfigGrpc.ConfigBlockingStub client = newConfigClient("localhost",50052);

        ClassLoader classLoader = getClass().getClassLoader();

        // path is test resources/config/example.json
        Input input = Input.newBuilder().
                setId("example.json").
                setName(Crawlers.REDIS).build();

        Output out = client.remove(input);

        ByteArrayInputStream bis = new ByteArrayInputStream(out.getData().toByteArray());
        ObjectInput in = new ObjectInputStream(bis);
        Object o = in.readObject();


        logger.info("!!!!!!!!!!!!1 " + o);
        assertEquals(true,o);
    }


    @Test
    public void TestGet() throws Exception {

        init();

        DataGrpc.DataBlockingStub client = newDataClient("localhost",50052);
        Input input = Input.newBuilder().
                setId("SOEJWEOJWOEJOSDJFOASDJFOSDFO2903870928734").
                setName(Crawlers.REDIS).build();

        Output out = client.get(input);

        ByteArrayInputStream bis = new ByteArrayInputStream(out.getData().toByteArray());
        ObjectInput in = new ObjectInputStream(bis);
        Object o = in.readObject();

        logger.info("!!!!!!!!!!!!!!!! " + o);

        assertEquals("SOEJWEOJWOEJOSDJFOASDJFOSDFO2903870928734",o);

    }


}