package com.loafle.overflow;

import com.loafle.overflow.rpc.*;
import io.grpc.ManagedChannel;
import io.grpc.ManagedChannelBuilder;
import io.grpc.StatusRuntimeException;
import org.junit.Test;

import java.util.concurrent.TimeUnit;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * Created by insanity on 17. 4. 12.
 */
public class TestClient {


    private static final Logger logger = Logger.getLogger(TestClient.class.getName());

    private final ManagedChannel channel;
    private final ConfigGrpc.ConfigBlockingStub configStub;
    private final DataGrpc.DataBlockingStub dataStub;

    public TestClient() {
        channel = ManagedChannelBuilder.forAddress("192.168.1.105", 50052).usePlaintext(true).build();
        configStub = ConfigGrpc.newBlockingStub(channel);
        dataStub = DataGrpc.newBlockingStub(channel);
    }


    public void shutdown() throws InterruptedException {
        channel.shutdown().awaitTermination(5, TimeUnit.SECONDS);
    }

    /** Say hello to server. */
    public void add() {
        InputAdd request = InputAdd.newBuilder().setData(null).setName(Crawlers.SNMP).build();
        Output response;
        try {
            response = configStub.add(request);
        } catch (StatusRuntimeException e) {
            logger.log(Level.WARNING, "RPC failed: {0}", e.getStatus());
            return;
        }
        logger.info("res : " + response.getData());
    }

    public void get() {
        Input request = Input.newBuilder().setId("test111").setName(Crawlers.SNMP).build();
        Output response;
        try {
            response = dataStub.get(request);
        } catch (StatusRuntimeException e) {
            logger.log(Level.WARNING, "RPC failed: {0}", e.getStatus());
            return;
        }
        logger.info("res : " + response.getStartDate());
    }

    @Test
    public void testRPCServer() {
        TestClient client = new TestClient();
        try {
            client.get();
        } catch (Exception e) {
            e.printStackTrace();
        }  finally {
            try {
                client.shutdown();
            } catch (InterruptedException e) {
                e.printStackTrace();
            }
        }
    }
}